@@ -26,11 +26,19 @@
 #include <fcntl.h>
 #include <stdio.h>
 #include <string.h>
+#if !(defined __MINGW32__ || defined _MSC_VER)
 #include <sys/ioctl.h>
+#else
+#include <sys/timeb.h> /* ftime() */
+#endif
 #include <sys/stat.h>
+#ifndef _MSC_VER
 #include <sys/time.h>
+#endif
 #include <sys/types.h>
+#ifdef __unix__
 #include <unistd.h>
+#endif
 
 #include "access.h"
 #include "debug.h"
@@ -43,14 +51,21 @@
 
 #ifdef HAVE_CYGWIN_NT
 
+#ifdef __CYGWIN__
 #include <w32api/windows.h>
+#else
+#include <windows.h>
+#endif
 #include "par_nt.h"
 
 
 static int
-init (struct parport_internal *port, int flags, int *capabilities)
-{
-  if (flags)
+init (struct parport *pport, int flags, int *capabilities)
+{ 
+  struct parport_internal *port = pport->priv;
+
+  /* Note: We can only ever provide exclusive access on NT. */
+  if (flags & ~F1284_EXCL) /* silently ignore F1284_EXCL - dbjh */
     return E1284_NOTAVAIL;
 
   port->fd = (int)CreateFile(port->device, GENERIC_READ | GENERIC_WRITE,
@@ -103,7 +118,7 @@ read_status (struct parport_internal *port)
           sizeof(ret), (LPDWORD)&dummy, NULL)))
       debugprintf("read_status: DeviceIoControl failed!\n");
 
-  return debug_display_status (ret ^ S1284_INVERTED);
+  return debug_display_status ((unsigned char)(ret ^ S1284_INVERTED));
 }
 
 static void
@@ -131,7 +146,7 @@ read_control (struct parport_internal *port)
 			    C1284_NAUTOFD |
 			    C1284_NINIT |
 			    C1284_NSELECTIN);
-  return port->ctr & rm;
+  return (port->ctr ^ C1284_INVERTED) & rm;
 }
 
 static void
@@ -143,10 +158,11 @@ write_control (struct parport_internal *port, unsigned char reg)
 			    C1284_NSELECTIN);
   if (reg & 0x20)
     {
-      printf ("use ieee1284_data_dir to change data line direction!\n");
+      /* Note: data direction not supported by LPT driver */
+      printf ("error: setting data dir is invalid in this mode!\n");
     }
 
-  raw_frob_control (port, wm, reg & wm);
+  raw_frob_control (port, wm, (unsigned char)(reg & wm));
 }
 
 static void
@@ -160,7 +176,8 @@ frob_control (struct parport_internal *port,
 			    C1284_NSELECTIN);
   if (mask & 0x20)
     {
-      printf ("use ieee1284_data_dir to change data line direction!\n");
+      /* Note: data direction not supported by LPT driver */
+      printf ("error: setting data dir is invalid in this mode!\n");
     }
 
   mask &= wm;
@@ -174,24 +191,39 @@ wait_status (struct parport_internal *port,
 	     struct timeval *timeout)
 {
   /* Simple-minded polling.  TODO: Use David Paschal's method for this. */
+#if !(defined __MINGW32__ || defined _MSC_VER)
   struct timeval deadline, now;
   gettimeofday (&deadline, NULL);
   deadline.tv_sec += timeout->tv_sec;
   deadline.tv_usec += timeout->tv_usec;
   deadline.tv_sec += deadline.tv_usec / 1000000;
   deadline.tv_usec %= 1000000;
+#else
+  struct timeb tb;
+  int deadline, now;
+  ftime (&tb);
+  deadline = tb.time * 1000 + tb.millitm +
+             timeout->tv_sec * 1000 + timeout->tv_usec / 1000;
+#endif
 
   do
     {
-      if ((debug_display_status (read_status (port)) & mask) == val)
+      if ((debug_display_status ((unsigned char)(read_status (port))) & mask) == val)
 	return E1284_OK;
 
       delay (IO_POLL_DELAY);
+#if !(defined __MINGW32__ || defined _MSC_VER)
       gettimeofday (&now, NULL);
     }
   while (now.tv_sec < deadline.tv_sec ||
-	 (now.tv_sec == deadline.tv_sec &&
+    (now.tv_sec == deadline.tv_sec &&
 	  now.tv_usec < deadline.tv_usec));
+#else
+      ftime (&tb);
+      now = tb.time * 1000 + tb.millitm;
+    }
+  while (now < deadline);
+#endif
 
   return E1284_TIMEDOUT;
 }
