@@ -26,7 +26,9 @@
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifndef _MSC_VER
 #include <sys/time.h>
+#endif
 #include <sys/types.h>
 #include <time.h>
 
@@ -44,11 +46,13 @@ timeofday (void)
 {
   static char str[100];
   struct timeval tod;
+#if !(defined __MINGW32__ || defined _MSC_VER)
   if (gettimeofday (&tod, NULL))
     {
       str[0] = '\0';
     }
   else
+#endif
     {
       struct tm *tm = localtime (&tod.tv_sec);
       char *p = str + strftime (str, 50, "%H:%M:%S.", tm);
@@ -69,7 +73,7 @@ debug_display_status (unsigned char st)
     goto out;
 
   last_status = st;
-  dprintf ("%s STATUS: %cnFault %cSelect %cPError %cnAck %cBusy\n",
+  debugprintf ("%s STATUS: %cnFault %cSelect %cPError %cnAck %cBusy\n",
 	   timeofday (),
 	   st & S1284_NFAULT ? ' ' : '!',
 	   st & S1284_SELECT ? ' ' : '!',
@@ -91,7 +95,7 @@ debug_display_control (unsigned char ct)
     goto out;
 
   soft_ctr = ct;
-  dprintf ("%s CONTROL: %cnStrobe %cnAutoFd %cnInit %cnSelectIn\n",
+  debugprintf ("%s CONTROL: %cnStrobe %cnAutoFd %cnInit %cnSelectIn\n",
 	   timeofday (),
 	   ct & C1284_NSTROBE ? ' ' : '!',
 	   ct & C1284_NAUTOFD ? ' ' : '!',
@@ -113,13 +117,14 @@ debug_frob_control (unsigned char mask, unsigned char val)
 }
 
 void
-dprintf (const char *fmt, ...)
+debugprintf (const char *fmt, ...)
 {
   if (!debugging_enabled)
     return;
 
   if (debugging_enabled == -1) {
     int dummy;
+    (void) dummy; /* warning remover for MinGW and VC++ */
 
     if (!getenv (ENVAR))
       {
@@ -127,12 +132,14 @@ dprintf (const char *fmt, ...)
 	return;
       }
 
+#if !(defined __MINGW32__ || defined _MSC_VER)
     /* Is stderr open? */
     if (fcntl (fileno (stderr), F_GETFL, &dummy) == -1 && errno == EBADF)
       {
 	debugging_enabled = 0;
 	return;
       }
+#endif
 
     debugging_enabled = 1;
   }
